@@ -1,9 +1,9 @@
-# -*- coding: future_annotations -*-
+# -*- coding: utf-8 -*-
 import sys
 
 from nbsafety.kernel.install import main
 
 
 # this is just a pointer to nbsafety.kernel.install
-if __name__ == '__main__':
+if __name__ == "__main__":
     sys.exit(main())
