@@ -28,13 +28,14 @@ from nbsafety.tracing.nbsafety_tracer import (
 
 
 logger = logging.getLogger(__name__)
+logger.setLevel(logging.WARNING)
 
 
 class PyccoloKernelSettings(NamedTuple):
     store_history: bool
 
 
-class SafeKernelHooks:
+class PyccoloKernelHooks:
     def after_init_class(self) -> None:
         ...
 
@@ -66,7 +67,7 @@ class OutputRecorder(pyc.BaseTracer):
         self.capture_output = self.capture_output_tee.__enter__()
 
 
-class PyccoloKernelMixin(SafeKernelHooks):
+class PyccoloKernelMixin(PyccoloKernelHooks):
     def __init__(self, **kwargs):
         self.settings: PyccoloKernelSettings = PyccoloKernelSettings(
             store_history=kwargs.pop("store_history", True)
@@ -351,7 +352,7 @@ class PyccoloKernelMixin(SafeKernelHooks):
         return ZMQKernel
 
 
-class SafeKernelMixin(singletons.SafeKernel, PyccoloKernelMixin):
+class SafeKernelBase(singletons.SafeKernel, PyccoloKernelMixin):
     def after_init_class(self) -> None:
         NotebookSafety.instance(use_comm=True)
 
@@ -376,6 +377,8 @@ class SafeKernelMixin(singletons.SafeKernel, PyccoloKernelMixin):
 
     def before_execute(self, cell_content: str) -> Optional[str]:
         nbs_ = singletons.nbs()
+        nbs_.test_and_clear_stale_usage_detected()
+        nbs_.test_and_clear_out_of_order_usage_detected_counter()
         if nbs_._saved_debug_message is not None:  # pragma: no cover
             logger.error(nbs_._saved_debug_message)
             nbs_._saved_debug_message = None
@@ -409,8 +412,6 @@ class SafeKernelMixin(singletons.SafeKernel, PyccoloKernelMixin):
                     used_out_of_order_counter,
                 )
                 return "pass"
-            else:
-                nbs_.test_and_clear_out_of_order_usage_detected_counter()
         return None
 
     def _handle_output(self):
@@ -419,7 +420,7 @@ class SafeKernelMixin(singletons.SafeKernel, PyccoloKernelMixin):
         if len(cell.history) >= 2:
             prev_cell = cells().from_timestamp(cell.history[-2])
         if (
-            singletons.nbs().test_and_clear_out_of_order_usage_detected_counter()
+            singletons.nbs().out_of_order_usage_detected_counter
             and prev_cell is not None
             and prev_cell.captured_output is not None
         ):
@@ -449,4 +450,4 @@ class SafeKernelMixin(singletons.SafeKernel, PyccoloKernelMixin):
             nbs_.set_exception_raised_during_execution(e)
 
 
-SafeKernel = SafeKernelMixin.make_zmq_kernel_class("SafeKernel")
+SafeKernel = SafeKernelBase.make_zmq_kernel_class("SafeKernel")
